<%@ page contentType="text/html; charset=UTF-8" import="java.sql.*, javax.sql.*, javax.naming.*,java.util.*" language="java" %><%!
	/*
		This code uses the linear least squares method to normalize GPS records.  An interative sample subset of the data (in this case 20 records) is shifted along the entire set in order to create a equation in the format of: y = beta1*x + beta0; where y = the normalized GPS reading, and x = the current time in seconds.  An accuracy threshold is also used, to only select records where the GPS accuracy reading is less than 11m.  This code assumes the same schema as point_data.sql.  The Java code creates a JavaScript array string of Google Map points; which is then printed to a map using the Google Map API.
	*/
	DataSource datasource = null;
	public void init(){
      	try {
			datasource = (DataSource) getServletContext().getAttribute("DBCPool");
			Class.forName("com.mysql.jdbc.Driver").newInstance();
		}catch (Exception e) {
			System.err.println(e.getMessage());
		}
    }
%><!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title>plot path</title>
<script type="text/javascript" src="http://maps.google.com/maps/api/js?sensor=false"></script>
<script type="text/javascript">
function myInit(){
var latlng = new google.maps.LatLng(50.397, -60.644);
	var myOptions = {
      zoom: 1,
      mapTypeId: google.maps.MapTypeId.ROADMAP,
	  streetViewControl: false,
	  center: latlng
    };
    var map = new google.maps.Map(document.getElementById("map_canvas"), myOptions);
<%
	if(request.getParameter("session_name") != null){
		Connection connection = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		ArrayList<Double> lats = new ArrayList<Double>(0);//these store the dynamic sample sets in a queue like manner (first in first out)
		ArrayList<Double> lons = new ArrayList<Double>(0);//The new data is added to the end of the arraylist, the old data is removed from the front
		ArrayList<Double> times = new ArrayList<Double>(0);//time is also required as we observe how the readings change over time
		double minTime = 0.0;
		try {
			connection = datasource.getConnection();
			StringBuilder sb = new StringBuilder("");
			//time stamps are large numbers, to avoid a series of mathematical operations on these large numbers, we just take the smallest time stamp (e.g. when the recording session started) and use the difference between that and the rest of the times.
			pstmt = connection.prepareStatement("select min(time_stamp) from point_data where session_name=? and horizontal_accuracy<11.0");
			pstmt.setString(1, request.getParameter("session_name").toString());
			rs=pstmt.executeQuery();
			if(rs.next()){
				minTime = rs.getDouble(1);
			}
			pstmt = connection.prepareStatement("select lat, lon, time_stamp from point_data where session_name=? and horizontal_accuracy<11.0 order by time_stamp asc");
			pstmt.setString(1, request.getParameter("session_name").toString());
			rs=pstmt.executeQuery();
			boolean first = true;
			while(rs.next()){
				//add new values to the end of the queue
				lats.add(new Double(rs.getDouble(1)));
				lons.add(new Double(rs.getDouble(2)));
				times.add(new Double(rs.getDouble(3)));
				if(lats.size()>20){//sets iterative sample size to 20
					lats.remove(0);//remove old values from the start of the queue if its full
					lons.remove(0);
					times.remove(0);
				}
				// solve for latitude
				double sumx = 0.0, sumy = 0.0, sumx2 = 0.0;
				for(int n = 0; n < lats.size(); n++) {
					sumx  += (times.get(n).doubleValue()-minTime);
					sumx2 += (times.get(n).doubleValue()-minTime) * (times.get(n).doubleValue()-minTime);
					sumy  += lats.get(n).doubleValue();
				}
				double xbar = sumx / lats.size();
				double ybar = sumy / lats.size();
				double xxbar = 0.0, yybar = 0.0, xybar = 0.0;
				for(int i = 0; i < lats.size(); i++) {
					xxbar += ((times.get(i).doubleValue()-minTime) - xbar) * ((times.get(i).doubleValue()-minTime) - xbar);
					yybar += (lats.get(i).doubleValue() - ybar) * (lats.get(i).doubleValue() - ybar);
					xybar += ((times.get(i).doubleValue()-minTime) - xbar) * (lats.get(i).doubleValue() - ybar);
				}
				double beta1 = xybar / xxbar;
				double beta0 = ybar - beta1 * xbar;
				double lat = (beta1*(times.get(times.size()-1).doubleValue()-minTime))+beta0;
				//solve for longitude
				sumx = 0.0; sumy = 0.0; sumx2 = 0.0;
				for(int n = 0; n < lats.size(); n++) {
					sumx  += (times.get(n).doubleValue()-minTime);
					sumx2 += (times.get(n).doubleValue()-minTime) * (times.get(n).doubleValue()-minTime);
					sumy  += lons.get(n).doubleValue();
				}
				xbar = sumx / lats.size();
				ybar = sumy / lats.size();
				xxbar = 0.0; yybar = 0.0; xybar = 0.0;
				for(int i = 0; i < lats.size(); i++) {
					xxbar += ((times.get(i).doubleValue()-minTime) - xbar) * ((times.get(i).doubleValue()-minTime) - xbar);
					yybar += (lons.get(i).doubleValue() - ybar) * (lons.get(i).doubleValue() - ybar);
					xybar += ((times.get(i).doubleValue()-minTime) - xbar) * (lons.get(i).doubleValue() - ybar);
				}
				beta1 = xybar / xxbar;
				beta0 = ybar - beta1 * xbar;
				double lon = (beta1*(times.get(times.size()-1).doubleValue()-minTime))+beta0;
				if(first){//solve NaN issue with first value that is output
					first = false;
					continue;	
				}
				sb.append("new google.maps.LatLng("+lat+", "+lon+"),");
			}
			sb.setLength(sb.length()-1);
			out.write("var path = ["+sb.toString()+"];var flightpath = new google.maps.Polyline({path:path, strokeColor: \"#FF0000\", strokeOpacity: 1.0, strokeWeight: 2, map: map});");
			connection.close();
		}catch (SQLException e) {
		  out.write("alert('error: "+e.toString()+"');");
		}finally {
		  try {if (rs != null) rs.close();} catch (SQLException e) {}
		  try {if (pstmt != null) pstmt.close();} catch (SQLException e) {}
		  try {if (connection != null) connection.close();} catch (SQLException e) {}
		}
	}
%>
}
</script>
</head>
<body onload="myInit();">
	<form method="post" action="plot_path_llsq.jsp">
    	<input type="text" size="20" name="session_name" />
        <input type="submit" value="Submit" />
    </form>
	<div id="map_canvas" style="width:600px;height:600px;"></div>
</body>
</html>